@@ -1,35 +1,38 @@
-import pytest
+# pylint: disable=R0201
+# pylint: disable=W0613
+
 from datetime import datetime, timedelta
 from uuid import uuid4
-from flask_login import login_user
-from sqlalchemy import desc, Integer
+import pytest
+from sqlalchemy import Integer
 from sqlalchemy_api_handler import ApiHandler, humanize
 from sqlalchemy_api_handler.bases.errors import JustBeforeActivityNotFound
-from sqlalchemy_api_handler.serialization import as_dict
 
 from tests.conftest import with_delete
 from api.models.activity import Activity
 from api.models.offer import Offer
 from api.models.stock import Stock
-from api.models.user import User
+
+
+def _assert_activity_match_model(patch, activity, offer):
+    assert activity.entityIdentifier == offer.activityIdentifier
+    assert patch.items() <= activity.datum.items()
+    assert patch.items() <= activity.patch.items()
+
+
+def assert_insert_activity_match_model(patch, activity, offer):
+    _assert_activity_match_model(patch, activity, offer)
+    assert activity.verb == 'insert'
+    assert activity.patch['id'] == humanize(offer.id)
+
+
+def assert_update_activity_match_model(patch, activity, offer):
+    _assert_activity_match_model(patch, activity, offer)
+    assert activity.verb == 'update'
+    assert activity.oldDatum['id'] == humanize(offer.id)
 
 
 class ActivateTest:
-    def test_models(self, app):
-        # When
-        models = ApiHandler.models()
-
-        # Then
-        assert Activity in models
-
-    def test_model_from_name(self, app):
-        # When
-        GotActivity = ApiHandler.model_from_name('Activity')
-
-        # Then
-        assert GotActivity == Activity
-
-
     def test_instance_an_activity(self, app):
         # Given
         offer_dict = { 'name': 'bar', 'type': 'foo' }
@@ -47,81 +50,6 @@ class ActivateTest:
         assert activity.patch['offerId'] == offer.humanizedId
 
 
-
-
-    @with_delete
-    def test_create_offer_saves_an_insert_activity(self, app):
-        # Given
-        offer_dict = { 'name': 'bar', 'type': 'foo' }
-        offer = Offer(**offer_dict)
-
-        # When
-        ApiHandler.save(offer)
-
-        # Then
-        all_activities = Activity.query.all()
-        offer_activities = offer.__activities__
-        insert_offer_activity = offer_activities[0]
-        insert_offer_activity_from_query = Activity.query \
-            .filter_by(entityIdentifier=offer.activityIdentifier) \
-            .one()
-        assert len(all_activities) == 1
-        assert len(offer_activities) == 1
-        assert insert_offer_activity_from_query == insert_offer_activity
-        assert offer.activityIdentifier == insert_offer_activity.entityIdentifier
-        assert insert_offer_activity.entityInsertedAt == offer.dateCreated
-        assert insert_offer_activity.oldDatum == {}
-        assert insert_offer_activity.transaction == None
-        assert insert_offer_activity.verb == 'insert'
-        assert offer_dict.items() <= insert_offer_activity.patch.items()
-        assert offer_dict.items() <= insert_offer_activity.datum.items()
-        assert insert_offer_activity.datum['id'] == humanize(offer.id)
-        assert insert_offer_activity.patch['id'] == humanize(offer.id)
-
-    @with_delete
-    def test_create_offer_with_login_user_saves_an_insert_activity_with_transaction(self, app):
-        # Given
-        offer_dict = { 'name': 'bar', 'type': 'foo' }
-        offer = Offer(**offer_dict)
-        user = User(email='fee@bar.com', firstName='fee', publicName='foo')
-        ApiHandler.save(user)
-        login_user(user)
-
-        # When
-        ApiHandler.save(offer)
-
-        # Then
-        all_activities = Activity.query.all()
-        offer_activities = offer.__activities__
-        insert_offer_activity = offer_activities[0]
-        assert len(all_activities) == 1
-        assert len(offer_activities) == 1
-        assert insert_offer_activity.transaction.actor.id == user.id
-
-    @with_delete
-    def test_modify_offer_saves_an_update_activity(self, app):
-        # Given
-        offer_dict = { 'name': 'bar', 'type': 'foo' }
-        offer = Offer(**offer_dict)
-        ApiHandler.save(offer)
-        modify_dict = { 'name': 'bor' }
-        offer.modify(modify_dict)
-
-        # When
-        ApiHandler.save(offer)
-
-        # Then
-        all_activities = Activity.query.all()
-        offer_activities = offer.__activities__
-        update_offer_activity = offer_activities[1]
-        assert len(all_activities) == 2
-        assert len(offer_activities) == 2
-        assert update_offer_activity.entityIdentifier == offer.activityIdentifier
-        assert update_offer_activity.verb == 'update'
-        assert {**offer_dict, **modify_dict}.items() <= update_offer_activity.datum.items()
-        assert modify_dict.items() == update_offer_activity.patch.items()
-        assert offer_dict.items() <= update_offer_activity.oldDatum.items()
-
     @with_delete
     def test_create_activity_on_not_existing_offer_saves_an_insert_activity(self, app):
         # Given
@@ -137,18 +65,13 @@ class ActivateTest:
 
         # Then
         all_activities = Activity.query.all()
-        offer = Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
+        offer = activity.entity
         offer_activities = offer.__activities__
-        insert_offer_activity = offer_activities[0]
+        assert offer == Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
         assert len(all_activities) == 1
         assert len(offer_activities) == 1
-        assert insert_offer_activity.entityInsertedAt == offer.dateCreated
-        assert insert_offer_activity.entityIdentifier == offer.activityIdentifier
-        assert insert_offer_activity.verb == 'insert'
-        assert patch.items() <= insert_offer_activity.datum.items()
-        assert patch.items() <= insert_offer_activity.patch.items()
-        assert insert_offer_activity.datum['id'] == humanize(offer.id)
-        assert insert_offer_activity.patch['id'] == humanize(offer.id)
+        assert_insert_activity_match_model(patch, offer_activities[0], offer)
+
 
     @with_delete
     def test_create_activity_on_not_existing_offers_saves_two_insert_activities(self, app):
@@ -173,29 +96,18 @@ class ActivateTest:
         all_activities = Activity.query.all()
         assert len(all_activities) == 2
 
-        offer1 = Offer.query.filter_by(activityIdentifier=offer1_activity_identifier).one()
+        offer1 = activity1.entity
+        assert offer1 == Offer.query.filter_by(activityIdentifier=offer1_activity_identifier).one()
         offer1_activities = offer1.__activities__
-        insert_offer1_activity = offer1_activities[0]
         assert len(offer1_activities) == 1
-        assert insert_offer1_activity.entityInsertedAt == offer1.dateCreated
-        assert insert_offer1_activity.entityIdentifier == offer1.activityIdentifier
-        assert insert_offer1_activity.verb == 'insert'
-        assert patch1.items() <= insert_offer1_activity.datum.items()
-        assert patch1.items() <= insert_offer1_activity.patch.items()
-        assert insert_offer1_activity.datum['id'] == humanize(offer1.id)
-        assert insert_offer1_activity.patch['id'] == humanize(offer1.id)
+        assert_insert_activity_match_model(patch1, offer1_activities[0], offer1)
 
-        offer2 = Offer.query.filter_by(activityIdentifier=offer2_activity_identifier).one()
+        offer2 = activity2.entity
+        assert offer2 == Offer.query.filter_by(activityIdentifier=offer2_activity_identifier).one()
         offer2_activities = offer2.__activities__
-        insert_offer2_activity = offer2_activities[0]
         assert len(offer2_activities) == 1
-        assert insert_offer2_activity.entityInsertedAt == offer2.dateCreated
-        assert insert_offer2_activity.entityIdentifier == offer2.activityIdentifier
-        assert insert_offer2_activity.verb == 'insert'
-        assert patch2.items() <= insert_offer2_activity.datum.items()
-        assert patch2.items() <= insert_offer2_activity.patch.items()
-        assert insert_offer2_activity.datum['id'] == humanize(offer2.id)
-        assert insert_offer2_activity.patch['id'] == humanize(offer2.id)
+        assert_insert_activity_match_model(patch2, offer2_activities[0], offer2)
+
 
     @with_delete
     def test_create_activities_on_existing_offer_saves_update_activities(self, app):
@@ -221,25 +133,17 @@ class ActivateTest:
         ApiHandler.activate(first_activity, second_activity, third_activity)
 
         # Then
-        offer = Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
+        offer = third_activity.entity
+        assert offer == Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
         all_activities = Activity.query.all()
         offer_activities = offer.__activities__
         assert len(all_activities) == 3
         assert len(offer_activities) == 3
-        assert offer_activities[0].entityInsertedAt == offer.dateCreated
-        assert offer_activities[0].entityIdentifier == offer.activityIdentifier
-        assert offer_activities[0].verb == 'insert'
-        assert offer_activities[0].id == offer_activities[0].id
-        assert offer_activities[1].entityInsertedAt == offer.dateCreated
-        assert offer_activities[1].entityIdentifier == offer.activityIdentifier
-        assert offer_activities[1].verb == 'update'
-        assert offer_activities[1].patch.items() == second_patch.items()
-        assert offer_activities[2].entityInsertedAt == offer.dateCreated
-        assert offer_activities[2].entityIdentifier == offer.activityIdentifier
-        assert offer_activities[2].verb == 'update'
-        assert offer_activities[2].patch.items() == third_patch.items()
         assert offer.name == 'bor'
         assert offer.type == 'fee'
+        assert_insert_activity_match_model(first_patch, offer_activities[0], offer)
+        assert_update_activity_match_model(second_patch, offer_activities[1], offer)
+        assert_update_activity_match_model(third_patch, offer_activities[2], offer)
 
     @with_delete
     def test_create_activity_stock_binds_relationship_with_offer(self, app):
@@ -295,13 +199,14 @@ class ActivateTest:
         ApiHandler.activate(stock_activity, offer_activity2)
 
         # Then
-        offer = Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
-        stock = Stock.query.filter_by(activityIdentifier=stock_activity_identifier).one()
+        offer = offer_activity2.entity
+        stock = stock_activity.entity
+        assert offer == Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
+        assert stock == Stock.query.filter_by(activityIdentifier=stock_activity_identifier).one()
         assert offer.activityIdentifier == offer_activity2.entityIdentifier
         assert stock.activityIdentifier == stock_activity.entityIdentifier
         assert stock.offerId == offer.id
 
-
     @with_delete
     def test_same_insert_activity_should_not_be_recorded_twice(self, app):
         # Given
@@ -326,13 +231,11 @@ class ActivateTest:
         assert offer.activityIdentifier == offer_activity1.entityIdentifier
         assert len(offer.__activities__) == 1
 
-
     @with_delete
     def test_same_insert_activity_should_not_be_recorded_twice_from_dict_edition(self, app):
         # Given
         offer_activity_identifier = uuid4()
         offer_patch = { 'name': 'bar', 'type': 'foo' }
-        date_created = datetime.utcnow()
         offer_activity1 = Activity(**{'modelName': 'Offer', 'dateCreated': '2021-03-31T14:18:52.618Z', 'localIdentifier': 'ec510dea-1087-4770-83be-91c2b91a7d05/2021-03-30T12:21:54.540Z', 'localDossierId': 'AEE8EHCP', 'entityIdentifier': str(offer_activity_identifier), 'patch': offer_patch})
         ApiHandler.activate(offer_activity1)
         duplicate_offer_activity = Activity(**{'modelName': 'Offer', 'dateCreated': '2021-03-31T14:18:52.618Z', 'localIdentifier': 'ec510dea-1087-4770-83be-91c2b91a7d05/2021-03-30T12:21:54.540Z', 'localDossierId': 'AEE8EHCP', 'entityIdentifier': str(offer_activity_identifier), 'patch': offer_patch})
@@ -342,10 +245,9 @@ class ActivateTest:
 
         # Then
         offer = Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
-        assert str(offer.activityIdentifier) == offer_activity1.entityIdentifier
+        assert offer.activityIdentifier == offer_activity1.entityIdentifier
         assert len(offer.__activities__) == 1
 
-
     @with_delete
     def test_same_update_activity_should_not_be_recorded_twice(self, app):
         # Given
@@ -391,14 +293,9 @@ class ActivateTest:
         ApiHandler.activate(activity)
 
         # Then
-        offer = Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
-        insert_offer_activity = offer.__activities__[0]
-        assert insert_offer_activity.entityIdentifier == offer.activityIdentifier
-        assert insert_offer_activity.verb == 'insert'
-        assert patch.items() <= insert_offer_activity.datum.items()
-        assert patch.items() <= insert_offer_activity.patch.items()
-        assert insert_offer_activity.datum['id'] == humanize(offer.id)
-        assert insert_offer_activity.patch['id'] == humanize(offer.id)
+        offer = activity.entity
+        assert offer == Offer.query.filter_by(activityIdentifier=offer_activity_identifier).one()
+        assert_insert_activity_match_model(patch, offer.__activities__[0], offer)
 
     @with_delete
     def test_create_delete_activity(self, app):
@@ -421,6 +318,33 @@ class ActivateTest:
         assert len(offers) == 0
         assert activity.entityIdentifier == offer.activityIdentifier
 
+    @with_delete
+    def test_create_delete_activity_with_previous_updated_activities(self, app):
+        # Given
+        offer = Offer(name='bar', type='foo')
+        ApiHandler.save(offer)
+
+        update_activity = Activity(dateCreated=datetime.utcnow(),
+                                   entityIdentifier=offer.activityIdentifier,
+                                   modelName='Offer',
+                                   patch={'name': 'bor'})
+
+        delete_activity = Activity(dateCreated=datetime.utcnow(),
+                                   entityIdentifier=offer.activityIdentifier,
+                                   modelName='Offer',
+                                   verb='delete')
+
+        # When
+        ApiHandler.activate(update_activity, delete_activity)
+
+        # Then
+        query_filter = (Activity.data['id'].astext.cast(Integer) == offer.id) & \
+                       (Activity.verb == 'delete')
+        activity = Activity.query.filter(query_filter).one()
+        offers = Offer.query.all()
+        assert len(offers) == 0
+        assert activity.entityIdentifier == offer.activityIdentifier
+
     @with_delete
     def test_raise_JustBeforeActivityNotFound_when_update_activity_date_before_insert_activity_date(self, app):
         # Given
@@ -441,5 +365,5 @@ class ActivateTest:
                                    tableName='offer')
 
         # When + Then
-        with pytest.raises(JustBeforeActivityNotFound) as error:
+        with pytest.raises(JustBeforeActivityNotFound):
             ApiHandler.activate(second_activity)
